@@ -2,6 +2,7 @@
 
 from Cocoa import *
 from PyObjCTools import AppHelper
+import sys
 
 class TinyTinyDocument(NSDocument):
     textView = objc.IBOutlet()
@@ -30,6 +31,16 @@ class TinyTinyDocument(NSDocument):
     def windowControllerDidLoadNib_(self, controller):
         if self.path:
             self.readFromUTF8(self.path)
+        else:
+            if hasattr(sys, 'maxint'):
+                maxint = sys.maxint
+                maxint_label = 'maxint'
+            else:
+                maxint = sys.maxsize
+                maxint_label = 'maxsize'
+
+            self.textView.setString_("Welcome to TinyTinyEdit in Python\nVersion: %s\nsys.%s: %d\nbyteorder: %s"%(
+                sys.version, maxint_label, maxint, sys.byteorder))
 
     def readFromUTF8(self, path):
         f = file(path)
