@@ -26,6 +26,8 @@ typedef void (*Py_FinalizePtr)(void);
 typedef PyObject *(*PySys_GetObjectPtr)(const char *);
 typedef int *(*PySys_SetArgvPtr)(int argc, char **argv);
 typedef PyObject *(*PyObject_GetAttrStringPtr)(PyObject *, const char *);
+typedef wchar_t* (*_Py_DecodeUTF8_surrogateescapePtr)(const char *s, ssize_t size);
+
 
 typedef CFTypeRef id;
 typedef const char * SEL;
@@ -98,6 +100,7 @@ USES(CFStringGetCString);
 USES(CFStringCreateByCombiningStrings);
 USES(CFDictionaryGetValue);
 USES(CFBooleanGetValue);
+USES(CFNumberGetValue);
 USES(CFStringCreateArrayBySeparatingStrings);
 USES(CFArrayAppendArray);
 USES(CFStringCreateByCombiningStrings);
@@ -215,6 +218,7 @@ static int bind_CoreFoundation(void) {
     LOOKUP(CFStringCreateByCombiningStrings);
     LOOKUP(CFDictionaryGetValue);
     LOOKUP(CFBooleanGetValue);
+    LOOKUP(CFNumberGetValue);
     LOOKUP(CFStringCreateArrayBySeparatingStrings);
     LOOKUP(CFArrayAppendArray);
     LOOKUP(CFStringCreateByCombiningStrings);
@@ -308,8 +312,6 @@ static int report_error(const char *error) {
 		    py2app_sel_getUid("init"));
     py2app_NSLog(py2app_CFSTR("%@"), py2app_CFSTR(error));
 
-    printf("%p\n", py2app_NSApplicationLoad);
-
     if (!py2app_NSApplicationLoad()) {
         py2app_NSLog(py2app_CFSTR("NSApplicationLoad() failed"));
     } else {
@@ -448,8 +450,8 @@ static void py2app_setPythonPath(void) {
     CFDictionaryRef options;
 
     paths = py2app_CFArrayCreateMutable(NULL, 0, py2app_kCFTypeArrayCallBacks);
-
     resDir = py2app_CFBundleCopyResourcesDirectoryURL(py2app_CFBundleGetMainBundle());
+
     resPath = pathFromURL(resDir);
     py2app_CFArrayAppendValue(paths, resPath);
     py2app_CFRelease(resPath);
@@ -479,6 +481,7 @@ static void py2app_setPythonPath(void) {
     options = py2app_getKey("PyOptions");
     if (options) {
         CFBooleanRef use_pythonpath;
+	CFNumberRef optimize;
         use_pythonpath = py2app_CFDictionaryGetValue(
             options, py2app_CFSTR("use_pythonpath"));
         if (use_pythonpath && py2app_CFBooleanGetValue(use_pythonpath)) {
@@ -496,13 +499,28 @@ static void py2app_setPythonPath(void) {
                 }
             }
         }
+
+	optimize = py2app_CFDictionaryGetValue(
+		options, py2app_CFSTR("optimize"));
+	if (optimize) {
+		int v = 0;
+		char buf[32];
+		py2app_CFNumberGetValue(optimize, kCFNumberIntType, &v);
+		snprintf(buf, 31, "%d", v);
+		setenv("PYTHONOPTIMIZE", buf, 1);
+	}
     }
 
     if (py2app_CFArrayGetCount(paths)) {
         resPath = py2app_CFStringCreateByCombiningStrings(NULL, paths, py2app_CFSTR(":"));
         setcfenv("PYTHONPATH", resPath);
         py2app_CFRelease(resPath);
+    } else {
+	 if (getenv("PYTHONPATH") != NULL) {
+	     unsetenv("PYTHONPATH");
+	 }
     }
+
     py2app_CFRelease(paths);
 }
 
@@ -850,6 +868,26 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     char* curlocale = NULL;
 
 
+    if (getenv("PYTHONOPTIMIZE") != NULL) {
+        unsetenv("PYTHONOPTIMIZE");
+    }
+    if (getenv("PYTHONDEBUG") != NULL) {
+        unsetenv("PYTHONDEBUG");
+    }
+    if (getenv("PYTHONDONTWRITEBYTECODE") != NULL) {
+        unsetenv("PYTHONDONTWRITEBYTECODE");
+    }
+    if (getenv("PYTHONIOENCODING") != NULL) {
+        unsetenv("PYTHONIOENCODING");
+    }
+    if (getenv("PYTHONDUMPREFS") != NULL) {
+        unsetenv("PYTHONDUMPREFS");
+    }
+    if (getenv("PYTHONMALLOCSTATS") != NULL) {
+        unsetenv("PYTHONMALLOCSTATS");
+    }
+
+
     if (!py2app_getApplicationName()) return report_error(ERR_NONAME);
     pyLocations = (CFArrayRef)py2app_getKey("PyRuntimeLocations");
     if (!pyLocations) return report_error(ERR_PYRUNTIMELOCATIONS);
@@ -867,6 +905,18 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     py2app_setPythonPath();
     setenv("ARGVZERO", argv[0], 1);
 
+    /* Clear unwanted environment variable that could be set
+     * by a PyObjC bundle in a parent process. Not clearing causes
+     * problems in PyObjC.
+     */
+    if (getenv("PYOBJC_BUNDLE_ADDRESS") != NULL) {
+        unsetenv("PYOBJC_BUNDLE_ADDRESS");
+    }
+    snprintf(buf, sizeof(buf)-1, "PYOBJC_BUNDLE_ADDRESS%ld", (long)getpid());
+    if (getenv(buf) != NULL) {
+        unsetenv(buf);
+    }
+
     mainScript = getMainScript();
     if (!mainScript) return report_error(ERR_NOPYTHONSCRIPT);
 
@@ -905,6 +955,7 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
 #if 0
     OPT_LOOKUP(Py_SetPath);
 #endif
+    OPT_LOOKUP(_Py_DecodeUTF8_surrogateescape);
     LOOKUP(PySys_SetObject);
 
 
@@ -921,7 +972,7 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
      * therefore explicitly set the locale. 
      *
      * Also set the LC_CTYPE environment variable because Py_Initialize
-     * reset the locale information using the environment :-(
+     * resets the locale information using the environment :-(
      */
     curlocale = setlocale(LC_ALL, NULL);
     if (curlocale != NULL) {
@@ -940,77 +991,17 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     setenv("LC_CTYPE", "en_US.UTF-8", 1);
 
     if (isPy3K) {
-#if 0
-	/*
-	 *  Py_SetPath is disabled for now, py2app works without
-	 *  it and the code below requires changes to work properly
-	 *  in all cases (crashes with alias and semi-standalone,
-	 *  and a hardcoded version string in the path string).
-	 */
-    	if (py2app_Py_SetPath != NULL) {
-		size_t len = (strlen(resource_path) * 3) + 256;
-		char* search_path = malloc(len);
-		wchar_t* w_search_path;
-
-		if (search_path == NULL) {
-			report_error("cannot allocate sys.path");
-			return -1;
-		}
-		/* FIXME: should use actual python version */
-		snprintf(search_path, len,
-			"%s/lib/python3.2:%s/lib/python32.zip:%s/lib/python3.2/lib-dynload",
-			resource_path, resource_path, resource_path);
-
-		w_search_path = (wchar_t*)malloc(len * sizeof(wchar_t));
-		if (search_path == NULL) {
-			report_error("cannot allocate wide sys.path");
-			return -1;
-		}
-
-		mbstowcs(w_search_path, search_path, len);
-		free(search_path); search_path = NULL;
-
-		py2app_Py_SetPath(w_search_path);
-		free(w_search_path);
-
-		/* sys.prefix and sys.exec_prefix aren't set yet */
-		
-	}
-#endif
-
     	wchar_t w_pythonInterpreter[PATH_MAX+1];
     	mbstowcs(w_pythonInterpreter, c_pythonInterpreter, PATH_MAX+1);
     	py2app_Py_SetProgramName((char*)w_pythonInterpreter);
 
 
-
     } else {
 	    py2app_Py_SetProgramName(c_pythonInterpreter);
     }
 
     py2app_Py_Initialize();
 
-#if 0
-    if (isPy3K && py2app_Py_SetPath) {
-	    PyObject* prefix = py2app_Py_BuildValue(
-		"s", resource_path);
-	    if (prefix == NULL) {
-        	rval = report_script_error(ERR_PYTHONEXCEPTION);
-		return rval;
-	    }
-
-	    if (py2app_PySys_SetObject("prefix", prefix) == -1) {
-        	rval = report_script_error(ERR_PYTHONEXCEPTION);
-		return rval;
-	    }
-	    if (py2app_PySys_SetObject("exec_prefix", prefix) == -1) {
-        	rval = report_script_error(ERR_PYTHONEXCEPTION);
-		return rval;
-	    }
-     }
-#endif
-
-
     /*
      * Reset the environment and locale information
      */
@@ -1029,12 +1020,24 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
         sizeof(c_mainScript), kCFStringEncodingUTF8);
     py2app_CFRelease(mainScript);
 
-    argv_new = alloca((argc + 1) * sizeof(char *));
-    argv_new[argc] = NULL;
-    argv_new[0] = c_mainScript;
-    memcpy(&argv_new[1], &argv[1], (argc - 1) * sizeof(char *));
-    py2app_PySys_SetArgv(argc, argv_new);
+    if (isPy3K) {
+       int i;
 
+       argv_new = alloca((argc+1) * sizeof(wchar_t));
+       argv_new[argc] = NULL;
+       argv_new[0] = (char*)py2app__Py_DecodeUTF8_surrogateescape(c_mainScript, strlen(c_mainScript));
+
+       for (i = 1; i < argc; i++) {
+	  argv_new[1] = (char*)py2app__Py_DecodeUTF8_surrogateescape(argv[i], strlen(argv[i]));
+       }
+
+    } else {
+       argv_new = alloca((argc + 1) * sizeof(char *));
+       argv_new[argc] = NULL;
+       argv_new[0] = c_mainScript;
+       memcpy(&argv_new[1], &argv[1], (argc - 1) * sizeof(char *));
+    }
+    py2app_PySys_SetArgv(argc, argv_new);
 
     mainScriptFile = fopen(c_mainScript, "r");
     rval = py2app_PyRun_SimpleFile(mainScriptFile, c_mainScript);
