@@ -3,8 +3,7 @@ def _run(*scripts):
     import os, sys, site
     sys.frozen = 'macosx_app'
     base = os.environ['RESOURCEPATH']
-    site.addsitedir(base)
-    site.addsitedir(os.path.join(base, 'Python', 'site-packages'))
+
     if not scripts:
         import __main__
     for script in scripts:
