@@ -31,7 +31,7 @@ from distutils.errors import *
 from altgraph.compat import *
 
 from modulegraph.find_modules import find_modules, parse_mf_results
-from modulegraph.modulegraph import SourceModule, Package
+from modulegraph.modulegraph import SourceModule, Package, Script
 from modulegraph import zipio
 
 import macholib.dyld
@@ -217,6 +217,8 @@ class py2app(Command):
          "Use argv emulation [disabled for plugins]. Does not work with python 3.x"),
         ("argv-inject=", None,
          "Inject some commands into the argv"),
+        ("emulate-shell-environment", None,
+         "Emulate the shell environment you get in a Terminal window"),
         ("use-pythonpath", None,
          "Allow PYTHONPATH to effect the interpreter's environment"),
         ('bdist-base=', 'b',
@@ -233,6 +235,8 @@ class py2app(Command):
          'Drop to pdb console after the module finding phase is complete'),
         ("debug-skip-macholib", None,
          "skip macholib phase (app will not be standalone!)"),
+        ("arch=", None, "set of architectures to use (fat, fat3, universal, intel, i386, ppc, x86_64; default is the set for the current python binary)"),
+
         ]
 
     boolean_options = [
@@ -251,6 +255,7 @@ class py2app(Command):
         "debug-skip-macholib",
         "graph",
         "prefer-ppc",
+        "emulate-shell-environment",
     ]
 
     def initialize_options (self):
@@ -261,12 +266,16 @@ class py2app(Command):
         self.graph = False
         self.no_zip = 0
         self.optimize = 0
+        if hasattr(sys, 'flags'):
+            self.optimize = sys.flags.optimize
+        self.arch = None
         self.strip = True
         self.no_strip = False
         self.iconfile = None
         self.extension = None
         self.alias = 0
         self.argv_emulation = 0
+        self.emulate_shell_environment = 0
         self.argv_inject = None
         self.no_chdir = 0
         self.site_packages = False
@@ -290,12 +299,6 @@ class py2app(Command):
         self.eggs = []
 
     def finalize_options (self):
-        if self.argv_emulation:
-            if sys.version_info[0] == 3:
-                raise DistutilsOptionError("argv-emulation is not supported on python 3.x")
-            elif sys.maxint > 2 ** 32:
-                raise DistutilsOptionError("argv-emulation is not supported for 64-bit executables")
-
         if not self.strip:
             self.no_strip = True
         elif self.no_strip:
@@ -383,6 +386,13 @@ class py2app(Command):
                 version = find_version(target.script)
             except ValueError:
                 pass
+
+        if not isinstance(version, (str, unicode)):
+            raise DistutilsOptionError("Version must be a string")
+
+        if sys.version_info[0] > 2 and isinstance(version, type('a'.encode('ascii'))):
+            raise DistutilsOptionError("Version must be a string")
+
         plist['CFBundleVersion'] = version
 
         name = self.distribution.get_name()
@@ -535,6 +545,7 @@ class py2app(Command):
                 site_packages=bool(self.site_packages),
                 alias=bool(self.alias),
                 argv_emulation=bool(self.argv_emulation),
+                emulate_shell_environment=bool(self.emulate_shell_environment),
                 no_chdir=bool(self.no_chdir),
                 optimize=self.optimize,
                 prefer_ppc=self.prefer_ppc,
@@ -677,7 +688,11 @@ class py2app(Command):
                 item.filename = fn
 
         py_files, extensions = parse_mf_results(mf)
-        py_files = list(py_files)
+
+        # Remove all top-level scripts from the list of python files,
+        # those get treated differently.
+        py_files = [ item for item in py_files if not isinstance(item, Script) ]
+
         extensions = list(extensions)
         return py_files, extensions
 
@@ -718,6 +733,9 @@ class py2app(Command):
             self.bdist_dir = os.path.join(bdist_base,
                 'python%s-standalone' % (sys.version[:3],), 'app')
 
+        if os.path.exists(self.bdist_dir):
+            shutil.rmtree(self.bdist_dir)
+        
         self.collect_dir = os.path.abspath(
             os.path.join(self.bdist_dir, "collect"))
         self.mkpath(self.collect_dir)
@@ -726,6 +744,8 @@ class py2app(Command):
         self.mkpath(self.temp_dir)
 
         self.dist_dir = os.path.abspath(self.dist_dir)
+        if os.path.exists(self.dist_dir):
+            shutil.rmtree(self.dist_dir)
         self.mkpath(self.dist_dir)
 
         self.lib_dir = os.path.join(self.bdist_dir,
@@ -820,7 +840,15 @@ class py2app(Command):
                     if rest_path.startswith('.'):
                         rest_path = rest_path[1:]
 
-                    self.copy_file(os.path.join(prefix, rest_path), execdst)
+                    if PYTHONFRAMEWORK:
+                        # When we're using a python framework bin/python refers to a stub executable
+                        # that we don't want use, we need the executable in Resources/Python.app
+                        dpath = os.path.join(prefix, 'Resources', 'Python.app', 'Contents', 'MacOS')
+                        self.copy_file(os.path.join(dpath, PYTHONFRAMEWORK), execdst)
+
+
+                    else:
+                        self.copy_file(os.path.join(prefix, rest_path), execdst)
 
                 else:
                     self.copy_file(sys.executable, execdst)
@@ -1079,6 +1107,7 @@ class py2app(Command):
 
     def initialize_prescripts(self):
         prescripts = []
+        prescripts.append('reset_sys_path')
         if 0 and sys.version_info[:2] >= (3, 2) and not self.alias:
             # Python 3.2 or later requires a more complicated
             # bootstrap
@@ -1087,9 +1116,15 @@ class py2app(Command):
         if self.site_packages or self.alias:
             prescripts.append('site_packages')
 
+        if is_system():
+            prescripts.append('system_path_extras')
+
         #if self.style == 'app':
         #    prescripts.append('setup_pkgresource')
 
+        if self.emulate_shell_environment:
+            prescripts.append('emulate_shell_environment')
+
         if self.argv_emulation and self.style == 'app':
             prescripts.append('argv_emulation')
             if u'CFBundleDocumentTypes' not in self.plist:
@@ -1159,6 +1194,7 @@ class py2app(Command):
             appname,
             plist=self.plist,
             extension=self.extension,
+            arch=self.arch,
         )
         appdir = fsencoding(appdir)
         resdir = os.path.join(appdir, 'Contents', 'Resources')
@@ -1181,6 +1217,7 @@ class py2app(Command):
             appname,
             plist=self.plist,
             extension=self.extension,
+            arch=self.arch,
         )
         appdir = fsencoding(appdir)
         resdir = os.path.join(appdir, 'Contents', 'Resources')
