@@ -2,9 +2,12 @@ import PIL
 import pygame
 import pydoc
 import docutils
-import sip
 import pyopengl
+import sip
 
 import numpy
 import scipy
 import matplotlib
+
+import virtualenv
+import pyside
