@@ -27,6 +27,7 @@ import shutil
 import time
 import os
 import signal
+import py2app
 
 DIR_NAME=os.path.dirname(os.path.abspath(__file__))
 
@@ -43,6 +44,7 @@ else:
 
 class TestBasicApp (unittest.TestCase):
     py2app_args = []
+    python_args = []
     app_dir = os.path.join(DIR_NAME, 'basic_app')
 
     # Basic setup code
@@ -51,13 +53,26 @@ class TestBasicApp (unittest.TestCase):
     # a base-class.
     @classmethod
     def setUpClass(cls):
+        env=os.environ.copy()
+        pp = os.path.dirname(os.path.dirname(py2app.__file__))
+        if 'PYTHONPATH' in env:
+            env['PYTHONPATH'] = pp + ':' + env['PYTHONPATH']
+        else:
+            env['PYTHONPATH'] = pp
+	
+        if 'LANG' not in env:
+            # Ensure that testing though SSH works
+            env['LANG'] = 'en_US.UTF-8'
+
         p = subprocess.Popen([
-                sys.executable,
+                sys.executable ] + cls.python_args + [
                     'setup.py', 'py2app'] + cls.py2app_args,
             cwd = cls.app_dir,
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT,
-            close_fds=True)
+            close_fds=True,
+            env=env
+            )
         lines = p.communicate()[0]
         if p.wait() != 0:
             print (lines)
@@ -180,6 +195,7 @@ class TestBasicAppUnicodePath (TestBasicApp):
     else:
         app_dir = os.path.join(DIR_NAME, 'basic_app ' + chr(2744))
 
+
     @classmethod
     def setUpClass(cls):
         try:
@@ -208,6 +224,28 @@ class TestBasicAliasAppUnicodePath (TestBasicAppUnicodePath):
 class TestBasicSemiStandaloneAppUnicodePath (TestBasicAppUnicodePath):
     py2app_args = [ '--semi-standalone', ]
 
+class TestOptimized1 (TestBasicApp):
+    py2app_args = [ '-O1' ]
+
+    def testIsOptimized(self):
+        p = self.start_app()
+
+        p.stdin.write('print(__debug__)\n'.encode('latin1'))
+        p.stdin.flush()
+        ln = p.stdout.readline()
+        self.assertEqual(ln.strip(), B("False"))
+
+class TestOptimized2 (TestBasicApp):
+    py2app_args = [ '-O2' ]
+
+    def testIsOptimized(self):
+        p = self.start_app()
+
+        p.stdin.write('print(__debug__)\n'.encode('latin1'))
+        p.stdin.flush()
+        ln = p.stdout.readline()
+        self.assertEqual(ln.strip(), B("False"))
+
 if __name__ == "__main__":
     unittest.main()
 
