@@ -14,6 +14,7 @@ import time
 import os
 import signal
 from distutils.sysconfig import get_config_var
+import py2app
 
 DIR_NAME=os.path.dirname(os.path.abspath(__file__))
 
@@ -45,13 +46,25 @@ class TestBasicPlugin (unittest.TestCase):
                 shutil.rmtree(os.path.join(cls.plugin_dir, 'dist'))
 
             cmd = [ sys.executable, 'setup.py', 'py2app'] + cls.py2app_args
+
+            env=os.environ.copy()
+            pp = os.path.dirname(os.path.dirname(py2app.__file__))
+            if 'PYTHONPATH' in env:
+                env['PYTHONPATH'] = pp + ':' + env['PYTHONPATH']
+            else:
+                env['PYTHONPATH'] = pp
+
+            if 'LANG' not in env:
+                env['LANG'] = 'en_US.UTF-8'
+
             
             p = subprocess.Popen(
                 cmd,
                 cwd = cls.plugin_dir,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.STDOUT,
-                close_fds=True)
+                close_fds=True,
+                env=env)
             lines = p.communicate()[0]
             if p.wait() != 0:
                 print (lines)
